package com.hand.app.pingAn.service.impl;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinapay.dto.HlsEbankCcbTransaction;
import com.chinapay.mapper.HlsEbankCcbTransactionMapper;
import com.chinapay.service.IHlsEbankCcbTransactionService;
import com.chinapay.service.impl.HlsEbankCcbTransactionServiceImpl;
import com.chinapay.util.StringUtil;
import com.hand.app.cmb.dto.Ld037;
import com.hand.app.cmb.service.impl.HclcCmbPolyServiceImpl;
import com.hand.app.cmb.service.impl.HlsEbankCmbTransqryServiceImpl;
import com.hand.app.cmb.util.Utils;
import com.hand.app.esignHclc.utils.SignHclcUtils;
import com.hand.app.pingAn.service.PingAnService;
import com.hand.app.pingAn.utils.ApiUtils;
import com.hand.app.zhongDengWang.dto.HlsWsRequests;
import com.hand.app.zhongDengWang.mapper.HlsWsRequestsMapper;
import com.hand.hap.core.IRequest;
import com.hand.hap.intergration.dto.HapInterfaceHeader;
import com.hand.hap.intergration.service.IHapInterfaceHeaderService;
import com.hand.hap.system.dto.ResponseData;
import com.pingan.openbank.api.sdk.common.http.HttpResult;
import com.pingan.openbank.api.sdk.entity.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.*;

@Service
public class PingAnServiceImpl implements PingAnService {

    @Autowired
    private HlsWsRequestsMapper hlsWsRequestsMapper;
    @Autowired
    private IHapInterfaceHeaderService headerService;
    @Autowired
    private IHlsEbankCcbTransactionService hlsEbankCcbTransactionService;

    private Logger logger = LoggerFactory.getLogger(getClass());


    private static Properties pro = null;

    static {
        String path = Thread.currentThread().getContextClassLoader().getResource("").getPath().substring(1);
        pro = Utils.loadProp(path + "pingAn.properties");
        if (Objects.isNull(pro)) {
            LoggerFactory.getLogger(HclcCmbPolyServiceImpl.class).error("配置文件初始化异常");
        }
    }
    /**
     * @param interfaceName
     * @param params
     * @return
     */
    @Override
    public ResponseData invoke(String interfaceName, JSONObject params) {
        ResponseData responseData = new ResponseData();
        JSONObject requestBody = ApiUtils.getBaseReqBody();
        requestBody.putAll(params);
        /*插入接口日志表*/
        HlsWsRequests hlsWsRequests = new HlsWsRequests();
        hlsWsRequests = createRequestLog(pro.getProperty("baseUrl")+"/V1.0/"+interfaceName, requestBody.toJSONString(), 1L);
        HttpResult result = ApiUtils.invoke(interfaceName, requestBody);
        if (Objects.nonNull(result)) {
            JSONObject res;
            if ("200".equals(result.getCode() + "")) {
                hlsWsRequests.setReturnStatus("s");
                responseData.setSuccess(true);
                // todo 不同接口返回不同数据,需要确认返回结果的格式
                switch (interfaceName) {
                    case "bedl/SubAcctMaintenance" :
                        hlsWsRequests.setFunctionName("pingAn_SubAcctMaintenance");
                        res = JSONObject.parseObject(result.getData());
                        // do something
                        ArrayList<String> strings = new ArrayList<>();
                        strings.add(res.getString("SubAccountNo"));
                        responseData.setRows(strings);
                        break;
                    case "bedl/InquiryIntoTheCurrentDetailsOfMainSubaccountAccountTwo" :
                        hlsWsRequests.setFunctionName("pingAn_recentDetailQuery");
                        res = JSONObject.parseObject(result.getData());
                        //do something
                        ArrayList<JSONObject> jsonObjects = new ArrayList<>();
                        jsonObjects.add(res);
                        responseData.setRows(jsonObjects);
                        break;
                    case "bedl/DetailReportQueryNew" :
                        hlsWsRequests.setFunctionName("pingAn_DetailReportQueryNew");
                        // 做文件下载
                        JSONObject arrRes = JSONArray.parseObject(result.getData());
                        ArrayList<JSONObject> fileInfo = new ArrayList<>();
                        fileInfo.add(arrRes);
                        responseData.setRows(fileInfo);
                        break;
                }


            } else {
                //请求失败
                hlsWsRequests.setReturnStatus("f");
                responseData.setSuccess(false);
            }
            hlsWsRequests.setResponseJson(result.getData());
            hlsWsRequests.setResponsedDate(new Date());
        }
        //更新日志
        hlsWsRequestsMapper.updateByPrimaryKeySelective(hlsWsRequests);
        return responseData;
    }


    public ResponseData detailNotify(IRequest iRequest, JSONObject data){
        ResponseData responseData = new ResponseData();
        if (Objects.nonNull(data) && data.size() > 0){
            // 存表
            HlsEbankCcbTransaction hlsEbankCcbTransaction = saveTransaction(iRequest,data);
            // 调用系统的接口服务，推送通知数据
            postNotify(JSONObject.toJSONString(hlsEbankCcbTransaction));
            responseData.setSuccess(true);
            responseData.setMessage("接收成功");
        }else{
            responseData.setSuccess(false);
            responseData.setMessage("接收失败");
        }
        return responseData;
    }


    private HlsWsRequests createRequestLog(String wsdlUrl, String requestClob, Long pkValue) {
        HlsWsRequests hlsWsRequests = new HlsWsRequests();
        hlsWsRequests.setRequestDate(new Date());
        hlsWsRequests.setRequestWsdlUrl(wsdlUrl);
        hlsWsRequests.setRequestJson(requestClob);
        hlsWsRequests.setStatusDate(new Date());
        hlsWsRequests.setParameterType("json");
        hlsWsRequests.setPkValue(pkValue);

        hlsWsRequestsMapper.insertSelective(hlsWsRequests);
        return hlsWsRequests;
    }

    private String postNotify(String params) {
        String sysName = "HCL_UPLOAD_FILE";
        // todo 待修改
        String apiName = "writeOffLd037";
        try {

            //初始化接口信息
            HapInterfaceHeader headerAndLineDTO = headerService.getHeaderAndLine(sysName, apiName);
            if (headerAndLineDTO == null) {
                logger.info("headerAndLineDTO is null， apiName:{}, sysName:{}", apiName, sysName);
                return "N";
            }

            String url = headerAndLineDTO.getDomainUrl() + headerAndLineDTO.getIftUrl();

            Map<String, String> headInfo = new HashMap<String, String>();
            headInfo.put("Content-Type", "application/x-www-form-urlencoded");

            JSONObject requestData = new JSONObject();
            requestData.put("requestData", params);

            //发送http请求
            SignHclcUtils signHclcUtils1 = new SignHclcUtils();
            String stringResult = signHclcUtils1.postString(url, requestData, headInfo, SignHclcUtils.APPLICATION_FORM_URLENCODED);

            //解析返回结果
            JSONObject jsonResult = JSONObject.parseObject(stringResult);
            if ("S".equalsIgnoreCase(jsonResult.getString("respCode"))) {
                return "Y";
            } else {
                return "N";
            }
        } catch (Exception e) {
            e.printStackTrace();
            return "N";
        }
    }

    private HlsEbankCcbTransaction saveTransaction(IRequest iRequest, JSONObject data) {
        HlsEbankCcbTransaction ccbTransaction = new HlsEbankCcbTransaction();
        ccbTransaction.setMerId(ApiUtils.AcctNo);
        ccbTransaction.setSttVchCardNo(data.getString("STT_VCH_CARD_NO"));
        ccbTransaction.setSttAcDate(data.getString("STT_AC_DATE"));
        ccbTransaction.setTranDate(data.getString("STT_TR_TIME"));
        ccbTransaction.setMerOrderNo(data.getString("STT_CONSUMER_SEQ_NO"));
        ccbTransaction.setCmbOrderId(data.getString("STT_JRN_NO"));
        ccbTransaction.setSttVchSeqNo(data.getString("STT_VCH_SEQ_NO"));
        ccbTransaction.setTranType(data.getString("STT_VCH_SIGN"));
        ccbTransaction.setAmount(data.getDouble("STT_VCH_AMT"));
        ccbTransaction.setSttAmtRmb(data.getDouble("STT_AMT_RMB"));
        ccbTransaction.setSttVchBal(data.getDouble("STT_VCH_BAL"));
        ccbTransaction.setSttVchRcvAcNo(data.getString("STT_VCH_RCV_AC_NO"));
        ccbTransaction.setSttVchRcvAcName(data.getString("STT_VCH_RCV_AC_NAME"));
        ccbTransaction.setSttVchRcvBkName(data.getString("STT_VCH_RCV_BK_NAME"));
        ccbTransaction.setSttVchRcvBkNo(data.getString("STT_VCH_RCV_BK_NO"));
        ccbTransaction.setSttVchRemark(data.getString("STT_VCH_REMARK"));
        ccbTransaction.setReturnCode(data.getString("STT_VCH_MARKCODE"));
        ccbTransaction.setReturnMsg(data.getString("STT_VCH_PART"));
        return hlsEbankCcbTransactionService.insertSelective(iRequest, ccbTransaction);
    }
}
